/****************************************************************/
/*                                                              */
/*                            Memory                            */
/*                                                              */
/*  Description: Fonctions de manipulation de la memoire        */
/*  Auteur:      LePhenixNoir                                   */
/*  Version:     3.0                                            */
/*  Date:        11.06.2014                                     */
/*  Fichier:     memory.c - Code des fonctions                  */
/*                                                              */
/****************************************************************/

#ifndef __FXLIB_H__
  #include "fxlib.h"
#endif

#ifndef _STDIO
  #include <stdio.h>
#endif

#ifndef _STDLIB
  #include <stdlib.h>
#endif

#ifndef _STRING
  #include <string.h>
#endif 

#include "memory.h"



FONTCHARACTER *memory_char2font(char *adresse)
{
  FONTCHARACTER *adr;
  int i;

  adr = calloc((strlen(adresse)+1),sizeof(FONTCHARACTER));
  for(i=0;i<strlen(adresse);i++) *(adr+i) = *(adresse+i);
  return adr;
}

int memory_createfile(char *adresse, int size)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int i = Bfile_CreateFile(adr,size);

  free(adr);
  return i;
}

int memory_createdir(char *adresse)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int i = Bfile_CreateDirectory(adr);

  free(adr);
  return 1;
}

int memory_openfile(char *adresse, int mode)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int i = Bfile_OpenFile(adr,mode);

  free(adr);
  return i;
}

int memory_deletefile(char *adresse)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int i = Bfile_DeleteFile(adr);

  free(adr);
  return i;
}

char **memory_alloc(int l)
{
  char **p = calloc(l,sizeof(char *));
  int i; for(i=0;i<l;i++) *(p+i) = calloc(20,1);
  return p;
}

void memory_free(char **p, int l)
{
  int i; for(i=0;i<l;i++) free(*(p+i));
  free(p);
}

int memory_find(char *adresse, char **files, int max)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  FONTCHARACTER found[30];
  FILE_INFO fileInfo;
  int searchHandle,i=1,j,x;

  if(x = Bfile_FindFirst(adr,&searchHandle,found,&fileInfo)) return 0;
  for(j=0;j<14 && *(found+j);j++) *(*files+j) = *(found+j);

  while(Bfile_FindNext(searchHandle,found,&fileInfo)==0 && i<max) {
    for(j=0;j<14 && *(found+j);j++) *(*(files+i)+j) = *(found+j);
    i++; }

  Bfile_FindClose(searchHandle);
  free(adr);
  return i;
}

int memory_exists(char *adresse)
{
  char *file[1];
  int x;

  *file = malloc(14); **file=0;
  x = memory_find(adresse,file,1);
  free(*file);

  return x!=0;
}

void *memory_load(char *adresse)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int handle, x, size;
  void *p;

  if((handle=Bfile_OpenFile(adr,_OPENMODE_READ))<0) { return NULL; }
  size = Bfile_GetFileSize(handle)+1;
  p = calloc(size,1);

  if(!p) {
    Bfile_CloseFile(handle); free(adr); return NULL; }
  if((x=Bfile_ReadFile(handle,p,size,0))<0) {
    Bfile_CloseFile(handle); free(adr); return NULL; }

  Bfile_CloseFile(handle);
  free(adr);
  return p;
}

int memory_save(char *adresse, void *data, int l)
{
  FONTCHARACTER *adr = memory_char2font(adresse);
  int x=0, handle;

  if(memory_exists(adresse)) x = Bfile_DeleteFile(adr);
  if(x<0) { free(adr); return x; }
  x = Bfile_CreateFile(adr,l+1);
  if(x<0) { free(adr); return x; }
  handle = Bfile_OpenFile(adr,0x02);
  if(handle<0) { free(adr); return handle; }
  x = memory_writefile(handle,data,l);
  if(x<0) { free(adr); return x; }
  memory_closefile(handle);

  free(adr);
  return 0;
}
